# Edit this configuration file to define what should be installed on
# your system.  Help is available in the configuration.nix(5) man page
# and in the NixOS manual (accessible by running ‘nixos-help’).

{ config, pkgs, lib, ... }:
{
  imports =
    [ # Include the results of the hardware scan.
      ./hardware-configuration.nix
    ];

  # VM
  virtualisation.vmVariant = {
  # following configuration is added only when building VM with build-vm
    virtualisation = {
      memorySize =  2048; # Use 2048MiB memory.
      cores = 3;         
    };
  };

  # Docker
  virtualisation.docker.enable = true;

  services.emacs.package = pkgs.emacs29-pgtk;
  services.emacs.enable = true;

  # optimize power usage
  powerManagement.enable = true;

  # optimize storage
  nix.optimise.automatic = true;

  # Bootloader.
  boot = {
    # systemd-boot
    loader.efi.canTouchEfiVariables = true;
    loader.efi.efiSysMountPoint = "/boot";
    loader.systemd-boot.enable = true;
    
    # plymouth
    plymouth = {
      enable = true;
      theme = "owl";
      themePackages = with pkgs; [
        (adi1090x-plymouth-themes.override {
          selected_themes = [ "owl" ];
        })
      ];
    };
    
    # Enable "Silent Boot"
    consoleLogLevel = 0;
    initrd.verbose = false;
    kernelParams = [
      "quiet"
      "splash"
      "boot.shell_on_fail"
      "loglevel=3"
      "rd.systemd.show_status=false"
      "rd.udev.log_level=3"
      "udev.log_priority=3"
      "video=DP-1:2560x1440@144" 
      "video=HDMI-A-1:1920x1080@75"
    ];
  };

  networking.hostName = "gunther";
  networking.networkmanager.enable = true;

  # Set your time zone.
  time.timeZone = "Europe/Berlin";
  time.hardwareClockInLocalTime = true;

  i18n.supportedLocales = [
    "en_US.UTF-8/UTF-8"
    "de_DE.UTF-8/UTF-8"
  ];

  # Select internationalisation properties.
  i18n.defaultLocale = "en_US.UTF-8";

  i18n.extraLocaleSettings = {
    LANG = "en_US.UTF-8";
    LC_ALL = "en_US.UTF-8";
    LANGUAGE = "en_US.UTF-8";
    LC_ADDRESS = "de_DE.UTF-8";
    LC_IDENTIFICATION = "de_DE.UTF-8";
    LC_MEASUREMENT = "de_DE.UTF-8";
    LC_MONETARY = "de_DE.UTF-8";
    LC_NAME = "de_DE.UTF-8";
    LC_NUMERIC = "de_DE.UTF-8";
    LC_PAPER = "de_DE.UTF-8";
    LC_TELEPHONE = "de_DE.UTF-8";
    LC_TIME = "de_DE.UTF-8";
  };

  services.greetd = {
    enable = true;
    settings = {
      default_session = {
        command = "${pkgs.greetd.tuigreet}/bin/tuigreet --time --remember --cmd Hyprland";
        user = "greeter";
      };
    };
  };

  # Configure console keymap
  console.keyMap = "us";

  # Enable CUPS to print documents.
  services.avahi = {
    enable = true;
    nssmdns4 = true;
    openFirewall = true;
    publish.enable = true;
    publish.userServices = true;
  };
  services.printing = {
    enable = true;
    drivers = [ pkgs.splix ];
    browsing = true;
    listenAddresses = [ "*:631" ];
    allowFrom = [ "all" ];
    defaultShared = true;
    extraConf = ''
      BrowseLocalProtocols all
    '';
    openFirewall = true;
  };

  # bluetooth
  hardware.bluetooth.enable = true;
  hardware.bluetooth.powerOnBoot = true;
  services.blueman.enable = true;

  # Enable sound with pipewire.
  security.rtkit.enable = true;
  services.pipewire = {
    enable = true;
    alsa.enable = true;
    alsa.support32Bit = true;
    pulse.enable = true;
  };

  # Define a user account.
  users.users.leonv = {
    isNormalUser = true;
    description = "Leon Vatthauer";
    extraGroups = [ "networkmanager" "wheel" "docker" ];
    initialPassword = "leonv"; # just for setup
  };

  # Allow unfree packages
  nixpkgs.config.allowUnfree = true;

  # List packages installed in system profile.
    environment.systemPackages = 
      with pkgs;
      [
        # for connecting to nas
        nfs-utils

        # some standards for convenience
        vim
        parted
        os-prober
        qpdfview
        swww

        # greeter
        greetd.gtkgreet

        # deps for hyprland / eww
        socat

        texliveFull
        adi1090x-plymouth-themes
      ];

  system.stateVersion = "24.05";

  # source zsh
  programs.zsh.enable = true;
  users.defaultUserShell = pkgs.zsh;

  nix.settings.substituters = [
    "https://aseipp-nix-cache.freetls.fastly.net" 
  ];

  # environment variables
  environment.sessionVariables = {
    ## disable logging when direnv changes
    DIRENV_LOG_FORMAT = [];
  };
  # fonts
  fonts = {
    packages = with pkgs; [
      nerd-fonts.hack
      nerd-fonts.dejavu-sans-mono
      nerd-fonts.droid-sans-mono
      nerd-fonts.noto
      mononoki
      # noto-fonts
      noto-fonts-cjk-sans
      noto-fonts-emoji
    ];
    enableDefaultPackages = true;
    fontconfig = {
      defaultFonts = {
        monospace = [ "BerkeleyMono Nerd Font Mono" ];
        sansSerif = [ "NotoSans Nerd Font" ];
        serif = [ "NotoSans Nerd Font" ];
        emoji = [ "Noto Color Emoji" ];
      };
    };
  };

  # enable flakes
  nix.settings.experimental-features = [ "nix-command" "flakes" ];

  # hyprland
  programs.hyprland.enable = true;

  # ssh
  programs.ssh.startAgent = true;
  programs.ssh.extraConfig = ''
    AddKeysToAgent yes
    IdentityFile ~/.ssh/git
    Host cipterm0.cip.cs.fau.de
      HostName cipterm0.cip.cs.fau.de
      User hy84coky
    '';

  # thunar
  programs.thunar = {
    enable = true;
    plugins = with pkgs.xfce; [
      thunar-archive-plugin
      thunar-volman
    ];
  };
  services.gvfs.enable = true; # Mount, trash, and others
  services.tumbler.enable = true; # thumbnail support for images
}
